@@ -8,13 +8,19 @@ from jupyter_dash import JupyterDash
 import dash_core_components as dcc
 import dash_html_components as html
 from dash.dependencies import Input, Output, State
+import plotly.express as px
+from dash_table import DataTable
+
+
 import webbrowser
 
+from macrodemos.common_components import app_model_parameter, app_table_headers, editable_cell_format, header_cell_format
+
+
 import numpy as np
 import pandas as pd
 
 
-
 # Esta parte controla asuntos de estética de la página
 external_stylesheets = ['https://codepen.io/chriddyp/pen/bWLwgP.css']
 
@@ -45,6 +51,22 @@ class SolowSwan:
         gy='Output growth rate'
     )
 
+    math_form = dict(
+        A=r'$A$',
+        s=r'$s$',
+        α=r'$\alpha$',
+        δ=r'$\delta$',
+        n=r'$n$',
+        k=r'$k_t$',
+        y=r'$y_t$',
+        sy=r'$sy_t$',
+        c=r'$c_t$',
+        Δk=r'$\Delta k_t$',
+        gy=r'${g_y}_t$'
+    )
+    endogenous = ['k','y','sy','c','Δk','gy']
+
+
     def __init__(self, A, s, α, δ, n):
         """
 
@@ -112,14 +134,27 @@ class SolowSwan:
         self.data = datos
 
     def plot_field(self, ser):
-        y = self.data[ser]
-        ss = self.steady_state[ser]
-        t = np.arange(y.size)
-        results = dict()
-        results['data'] = [{'x': t, 'y': y, 'type': 'line','name': 'scenario'},
-                           {'x': t, 'y': ss + np.zeros_like(y), 'type':'line','name':'baseline'}]
-        results['layout'] = {'title': self.names[ser], 'legend':{'orientation':'h'}}
-        return results
+        df = self.data[[ser]].rename(columns={ser: 'Alternative'})
+        df['Baseline'] = self.steady_state[ser]
+        fig = px.line(df,
+                      y=df.columns,
+                      title=self.names[ser],
+                      template='simple_white'
+                      )
+        fig.update_layout(legend_orientation = 'h',
+                          xaxis_title = '',
+                          yaxis_title = '')
+
+        fig.add_annotation(x=0.1,
+                           y=0.9,
+                           xref='paper',
+                           yref='paper',
+                           showarrow=False,
+                           text= self.math_form[ser],
+                           font=dict(size=30)
+                           )
+        return fig
+
 
 
 #=======================================================================================================================
@@ -130,13 +165,6 @@ class SolowSwan:
 mathjax = ["https://cdnjs.cloudflare.com/ajax/libs/mathjax/2.7.4/MathJax.js?config=TeX-MML-AM_CHTML"]  # to display math
 omega = np.linspace(0, np.pi, 121)  # frequencies for spectral plot
 
-def app_model_parameter(label, base_value, scen_value): # to make input parameter
-    txt = html.Th(label)
-    param1 = html.Th(dcc.Input(id='base_' + label, type='text', value=base_value, size='10'))
-    param2 = html.Th(dcc.Input(id='scen_' + label, type='text', value=scen_value, size='10'))
-    return html.Tr([txt, param1, param2])
-
-
 
 app = JupyterDash(__name__,external_stylesheets=external_stylesheets, external_scripts=mathjax)
 
@@ -148,6 +176,7 @@ app.layout = html.Div(children=[
             style={'textAlign': 'center', 'color': colors['text']}),
     html.Div(children=[html.H4("Parameters"),
                        html.Table(children=[
+                           app_table_headers(['  ', 'Baseline', 'Alternative']),
                            app_model_parameter('α', 0.35, 0.35),
                            app_model_parameter('δ', 0.06, 0.06),
                            app_model_parameter('n', 0.02, 0.02)]
@@ -155,16 +184,33 @@ app.layout = html.Div(children=[
                        html.Hr(),
                        html.H4("Exogenous variables"),
                        html.Table(children=[
+                           app_table_headers(['  ', 'Baseline', 'Alternative']),
                            app_model_parameter('A', 1.0, 1.0),
                            app_model_parameter('s', 0.2, 0.2)]
                        ),
                        html.Hr(),
+                       html.H4("Steady state"),
+                       html.Div(id='output-data-upload',
+                                children=[
+                                    DataTable(
+                                        id='table-transition-matrix',
+                                        columns=([{'id': p, 'name': p} for p in ['Variable', 'Baseline', 'Alternative', '% change']]),
+                                        editable=False,
+                                        #style_table={'width': '75%'},
+                                        style_cell=editable_cell_format,
+                                        style_header=header_cell_format
+                                    )
+                                ],
+                                ),
+                       html.Hr(),
                        html.H4("Figure parameter"),
                        html.Table(children=[
                            html.Tr([html.Th('Number of periods'), dcc.Input(id='horizon', type='text', value=60, size='10')])]
                        ),
                        html.Button('PLOT', id='execute',style={'textAlign': 'center', 'backgroundColor': colors['buttons']}),
-                       html.P('Solow-Swan-DEMO', style={'textAlign': 'center', 'color': colors['text'],'marginTop':250}),
+                       html.H5('Solow-Swan-DEMO', style={'textAlign': 'center', 'color': colors['text'],'marginTop':100}),
+                       html.P('Based on chapter 9 of Bongers, Gómez and Torres (2019) Introducción a la Macroeconomía Computacional. Vernon Press.',
+                              style={'textAlign': 'left', 'color': colors['text'],'marginTop':20}),
                        ],
              style={'textAlign': 'center', 'color': colors['controls'], 'width': '25%', 'display': 'inline-block'}),
 
@@ -172,23 +218,23 @@ app.layout = html.Div(children=[
              children=[
                  # --PLOT 1a:  capital stock----------------------------------------------------------------------------
                  dcc.Graph(id='plot-capital',
-                           style={'width': '33%', 'float': 'left', 'display': 'inline-block'}),
+                           style={'width': '49%', 'float': 'left', 'display': 'inline-block'}),
                  #--PLOT 1b:  output per capita ------------------------------------------------------------------------------
                  dcc.Graph(id='plot-output',
-                           style={'width': '33%', 'float': 'left', 'display': 'inline-block'}),
+                           style={'width': '49%', 'float': 'left', 'display': 'inline-block'}),
                  #--PLOT 1c: savings---------------------------------------------------------------------------
                  dcc.Graph(id='plot-savings',
-                           style={'width': '33%', 'float': 'left', 'display': 'inline-block'}),
+                           style={'width': '49%', 'float': 'left', 'display': 'inline-block'}),
                  # --PLOT 2a:  consumption----------------------------------------------------------------
                  dcc.Graph(id='plot-consumption',
-                           style={'width': '33%', 'float': 'left', 'display': 'inline-block'}),
+                           style={'width': '49%', 'float': 'left', 'display': 'inline-block'}),
                  #--PLOT 2b: change in capital--------------------------------------------------------------------------
                  dcc.Graph(id='plot-delta-capital',
-                           style={'width': '33%', 'float': 'left', 'display': 'inline-block'}),
+                           style={'width': '49%', 'float': 'left', 'display': 'inline-block'}),
                  #--PLOT 2c:  Growth rate -------------------------------------------------------------------------------
                  dcc.Graph(id='plot-growth',
-                           style={'width': '33%', 'float': 'left', 'display': 'inline-block'}),
-                 # --PLOT 3b:  IMPULSE RESPONSE FUNCTION----------------------------------------------------------------
+                           style={'width': '49%', 'float': 'left', 'display': 'inline-block'}),
+                 # --Link to my website----------------------------------------------------------------
                  html.A(children='Randall Romero Aguilar', href='http://randall-romero.com', style={'textAlign': 'center', 'color': colors['text'],'marginBottom':60}),
              ]),
 ],
@@ -202,6 +248,7 @@ app.layout = html.Div(children=[
      Output('plot-consumption', 'figure'),
      Output('plot-delta-capital', 'figure'),
      Output('plot-growth', 'figure'),
+     Output('table-transition-matrix', 'data')
      ],
     [Input('execute', 'n_clicks')],
     [State('horizon','value'),
@@ -218,7 +265,16 @@ app.layout = html.Div(children=[
 def update_ARMA_parameters(n_clicks,T, α, δ, n, A, s, α1, δ1, n1, A1, s1):
     modelo = SolowSwan(*[float(xx) for xx in (A, s, α, δ, n)])
     modelo.shock(int(T), *[float(xx) for xx in [A1, s1, α1, δ1, n1]])
-    return [modelo.plot_field(ser) for ser in ['k','y','sy','c','Δk','gy']]
+
+    modelo1= SolowSwan(*[float(xx) for xx in (A1, s1, α1, δ1, n1)])
+    df = pd.DataFrame({'Baseline': [v for v in modelo.steady_state.values()],
+                      'Alternative': [v for v in modelo1.steady_state.values()]},
+                      index=modelo.steady_state.keys()
+                      )
+    df['% change'] = 100 * (df['Alternative'] / df['Baseline'] - 1)
+    df = df.round(3).loc[modelo.endogenous]
+    df.index.name = 'Variable'
+    return *[modelo.plot_field(ser) for ser in modelo.endogenous], df.reset_index().to_dict('records')
 
 
 
