@@ -29,12 +29,15 @@ import dash_html_components as html
 from dash.dependencies import Input, Output, State
 import plotly.graph_objects as go
 import plotly.express as px
+
+from macrodemos.common_components import app_parameter_row, app_choose_parameter
+
 from statsmodels.formula.api import ols
-from statsmodels.tsa.filters.cf_filter import cffilter
 import webbrowser
 
 import numpy as np
 import pandas as pd
+from scipy.linalg import toeplitz
 
 import requests
 import bs4 as bs
@@ -158,7 +161,7 @@ def get_data(country, datatype):
 """==========================================================================================================
 This block defines all filters
 """
-def HP(x: pd.DataFrame, lambda_: float):
+def HP_filter(x: pd.DataFrame, lambda_: float):
     """
     Returns the Hodrick-Prescott trend
     :param x: data
@@ -174,7 +177,7 @@ def HP(x: pd.DataFrame, lambda_: float):
     trend = pd.DataFrame(np.exp(trend), index=x.index, columns=x.columns)
     return trend, 100*(x/trend - 1)
 
-def BKcycle(x: pd.DataFrame, K:int=8, pL=6, pH=32):
+def BK_filter(x: pd.DataFrame, K:int=8, pL=6, pH=32):
     """
     Baxter-King filter
 
@@ -186,9 +189,9 @@ def BKcycle(x: pd.DataFrame, K:int=8, pL=6, pH=32):
     """
     pL, pH = (pH, pL) if pL > pH else (pL, pH)
     wh, wl = 2 * np.pi / np.array([pL, pH])
-    h = np.arange(-K, K+1)  # lags
+    h = np.arange(1, K+1)  # lags   
     beta = (np.sin(h*wh) - np.sin(h*wl)) / (h*np.pi)
-    beta[K] = (wh-wl)/np.pi # fix division by zero
+    beta = np.r_[beta[::-1], (wh-wl)/np.pi, beta]
     beta -= beta.mean()  # weights add-up to zero
     m = beta.size # = 2*K + 1
 
@@ -197,24 +200,52 @@ def BKcycle(x: pd.DataFrame, K:int=8, pL=6, pH=32):
     return np.exp(xtrend), 100*xcycle
 
 
-def CFcycle(x: pd.DataFrame, pL=6, pH=32):
+def CF_filter(x: pd.DataFrame, pL=6, pH=32, undrift=True):
     """
     Christiano Fitzgerald asymmetric, random walk filter.
 
     :param x: data to be filtered
     :param pL: cut-off low-periodicity, integer
     :param pH: cut-off high-periodicity, integer
+    :param undrift: whether to remove the drift, bool
     :return: two pd.DataFrame with trend and cycle data
+
+    Note: ported from Matlab code posted by authors at https://www.frbatlanta.org/cqer/research/bpf.aspx
     """
+    T = x.shape[0]
+    t = np.arange(T)
+    t1 = t[1:]
 
-    cycle, trend = cffilter(np.log(x), pL, pH, drift=False)
-    cycle.columns = x.columns
-    trend.columns = x.columns
+    pL, pH = (pH, pL) if pL > pH else (pL, pH)
+    wh, wl = 2 * np.pi / np.array([pL, pH])
 
-    return np.exp(trend), 100*cycle
+    # remove the drift
+    logx = np.log(x)
+    if undrift:
+        drift = (logx.iloc[-1] - logx.iloc[0]) / (T - 1)
+        Xun = logx - np.outer(t, drift)
+    else:
+        Xun = logx
+
+    # create ideal betas then construct AA matrix
+    bnot = (wh - wl) / np.pi
+    bhat = bnot / 2
+
+    beta = np.r_[bnot, (np.sin(t1 * wh) - np.sin(t1 * wl)) / (t1 * np.pi)]
+    AA = toeplitz(beta, beta)
+    AA[0, 0] = AA[-1, -1] = bhat
+
+    AA[1:, 0] = AA[0, 0] - beta[:-1].cumsum()
+    AA[:-1, -1] = np.flip(AA[:-1, 0] - beta[:-1])
+
+    xcycle = AA @ Xun
+    xcycle.index = x.index
+
+    xtrend = logx - xcycle
+    return np.exp(xtrend), 100 * xcycle
 
 
-def Hamilton(y: pd.DataFrame, h=8, p=4):
+def Hamilton_filter(y: pd.DataFrame, h=8, p=4):
     """
     Calculates the cyclical component based on 2-year-ahead forecast error
     from linear regression.
@@ -240,10 +271,10 @@ def Hamilton(y: pd.DataFrame, h=8, p=4):
     return np.exp(trend), 100*cycle
 
 
-FILTERS = {'HP': HP,
-           'BK': BKcycle,
-           'CF': CFcycle,
-           'H': Hamilton}
+FILTERS = {'HP': HP_filter,
+           'BK': BK_filter,
+           'CF': CF_filter,
+           'H': Hamilton_filter}
 
 
 
@@ -253,17 +284,15 @@ This block contains auxiliary functions to make it easier to build the app layou
 
 mathjax = ["https://cdnjs.cloudflare.com/ajax/libs/mathjax/2.7.4/MathJax.js?config=TeX-MML-AM_CHTML"]
 
-def app_choose_parameter(label, appid, list_of_options, value):
-    options = [{'label': x, 'value': x} for x in list_of_options]
-    return html.Tr([html.Th(label), html.Th(dcc.Dropdown(id=appid, options=options, value=value),style={'width': '100%'})])
 
-def parse_parameters(lambda_, K, pL, pH, cfpL, cfpH, h, p):
+def parse_parameters(lambda_, K, pL, pH, cfpL, cfpH, undrift, h, p):
     K, h, p = (int(x) for x in [K, h, p])
     lambda_, pL, pH, cfpL, cfpH = (float(x) for x in [lambda_, pL, pH, cfpL, cfpH])
+    undrift = undrift=='Yes'
     parameters = {
         'HP': (lambda_, ),
         'BK': (K, pL, pH),
-        'CF': (cfpL, cfpH),
+        'CF': (cfpL, cfpH, undrift),
         'H': (h, p),
     }
     return parameters
@@ -279,13 +308,6 @@ def add_filtered_data_to_plots(filter, trend_plot, cycle_plot, comovement_data,
     return None
 
 
-def app_parameter(label, appid, inputtype, value, size='20'):
-    return html.Th(label), html.Th(dcc.Input(id=appid, type=inputtype, value=value, size=size))
-
-
-def app_parameter_row(label, appid, inputtype, value, size='20'):
-    return html.Tr([*app_parameter(label, appid, inputtype, value, size)])
-
 
 """==========================================================================================================
 This block builds the app layout
@@ -327,7 +349,12 @@ app.layout = html.Div(children=[
                            dcc.Tab(label='Christiano-Fitzgerald', style={'padding-left': '0.4em'},
                                    children=[html.Table(
                                        children=[app_parameter_row('p_L', 'cfpL', 'text', 6, '6'),
-                                                 app_parameter_row('p_H', 'cfpH', 'text', 32, '6')],
+                                                 app_parameter_row('p_H', 'cfpH', 'text', 32, '6'),
+                                                 html.Tr([html.Th('Remove drift'),
+                                                          html.Th(dcc.RadioItems(id='undrift',
+                                                                                 options=[{'label': 'Yes', 'value': 'Yes'}, {'label': 'No', 'value': 'No'}],
+                                                                                 value='Yes'))]),
+                                                 ],
                                    )]),
                            dcc.Tab(label='Hamilton', style={'padding-left': '0.4em'},
                                    children=[html.Table(
@@ -458,13 +485,14 @@ def show_indicators(country, datatype):
      State('pH', 'value'),
      State('cfpL', 'value'),
      State('cfpH', 'value'),
+     State('undrift', 'value'),
      State('h', 'value'),
      State('p', 'value'),
      State('country', 'value')
      ]
 )
-def update_plots(n_clicks, indicator, whichFilters, lambda_, K, pL, pH, cfpL, cfpH, h, p, country):
-    params = parse_parameters(lambda_, K, pL, pH, cfpL, cfpH, h, p)
+def update_plots(n_clicks, indicator, whichFilters, lambda_, K, pL, pH, cfpL, cfpH, undrift, h, p, country):
+    params = parse_parameters(lambda_, K, pL, pH, cfpL, cfpH, undrift, h, p)
     trend = go.Figure()
     cycle = go.Figure()
     comovement_data = dict()
@@ -501,10 +529,6 @@ This block executes the app
 """
 
 def filters_demo(colab=True):
-
-    print('NOT READY: THIS DEMO WILL BE READY FOR NEXT RELEASE')
-    return None
-
     if colab:
         app.run_server(mode='external')
     else:
